-------------------------------------------------------------------------------
--                G  O  B  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

-- Copied from Blizzard_TradeSkillRecipeList.lua
local LEARNED_TAB   = 1;
local UNLEARNED_TAB = 2;

-- Mapping of spell ID to trade skill ID
local gatherSpellSkill =
{
  -- Engineering
  [ 49383] = 202,
  -- Fishing
  [  7620] = 356,
  [  7731] = 356,
  [  7732] = 356,
  [ 18248] = 356,
  [ 33095] = 356,
  [ 51294] = 356,
  [ 88868] = 356,
  [110410] = 356,
  [131474] = 356,
  [131476] = 356,
  [131490] = 356,
  [158743] = 356,
  -- Herbalism
  [  2366] = 182,
  [  2368] = 182,
  [  3570] = 182,
  [ 11993] = 182,
  [ 28695] = 182,
  [ 50300] = 182,
  [ 74519] = 182,
  [110413] = 182,
  [158745] = 182,
  [195114] = 182,
  -- Mining
  [  2575] = 186,
  [  2576] = 186,
  [  3564] = 186,
  [ 10248] = 186,
  [ 29354] = 186,
  [ 50310] = 186,
  [ 74517] = 186,
  [102161] = 186,
  [158754] = 186,
  [195122] = 186,
  -- Skinning
  [  8613] = 393,
  [  8617] = 393,
  [  8618] = 393,
  [ 10768] = 393,
  [ 32678] = 393,
  [ 50305] = 393,
  [ 74522] = 393,
  [102216] = 393,
  [158756] = 393,
  [195125] = 393,
};

-- Info about the gathering skills
local gatherSkillInfo =
{
  [202] = -- Engineering
  {
    index   = 0,
    tooltip = L["GatherEngineering"];
  },
  [356] = -- Fishing
  {
    index   = 0,
    tooltip = ""
  },
  [182] = -- Herbalism
  {
    index   = 0,
    tooltip = L["GatherHerbalism"];
  },
  [186] = -- Mining
  {
    index   = 0,
    tooltip = L["GatherMining"];
  },
  [393] = -- Skinning
  {
    index   = 0,
    tooltip = L["GatherSkinnable"];
  },
};

local eventFrame;

--
-- These value have been copied from 'Blizzard_TradeSkillUI.lua' because the
-- Trade Skill panel is implemented as an on-demand addon.
--
local tradeSkillTypeInfo =
{
  ["trivial"] = {r = 0.50, g = 0.50, b = 0.50},
  ["easy"]    = {r = 0.25, g = 0.75, b = 0.25},
  ["medium"]  = {r = 1.00, g = 1.00, b = 0.00},
  ["optimal"] = {r = 1.00, g = 0.50, b = 0.25},
};

local tradeSkillInfo =
{
  buttons        = {},
  buttonsBg      = nil,
  textures       = {},
  history        = nil,
  historyEnabled = true,
  recipes        = {},
  products       = {},
  resultTexture  = nil,
};

local tradeSkillHistory = setmetatable({},
{
  __index = function(tbl, key)
    tbl[key] = NigglesHistoryClass:New();
    return tbl[key];
  end
});

-------------------------------------------------------------------------------
--             L  O  C  A  L     D  E  F  I  N  I  T  I  O  N  S
-------------------------------------------------------------------------------

local recipeIsPassive;
local tradeSkillInfoUpdate;
local tradeSkillFrameOnShow;
local tradeSkillHistoryBtnEnable;
local tradeSkillHistoryBtnOnClick;
local tradeSkillRecipeShow;
local tradeSkillTypeFromTooltip;

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function hooked to 'TradeSkillFrame.Details:RefreshDisplay()'. It
-- shows/hides the textures used to indicate recipe reagents that can be
-- made by the current trade skill and character. It also adds the selected
-- recipe to the history.
--
local function recipeDetailsOnRefresh(self)
  -- Local Variables
  local numReagents;
  local reagentBtns = TradeSkillFrame.DetailsFrame.Contents.Reagents;
  local reagentLink;
  local reagentId;
  local recipeId = self.selectedRecipeID;
  local strfind = string.find;
  local textures = tradeSkillInfo.textures;

  -- Check there is a recipe displayed
  numReagents = C_TradeSkillUI.GetRecipeNumReagents(recipeId or 0);
  if ((TradeSkillFrame:IsShown()) and
      (recipeId ~= nil) and
      (numReagents ~= nil)) then
    -- Show/Hide the textures that indicate a reagent can be created
    for idx = 1, numReagents do
      -- Get the reagent's ID
      reagentLink = C_TradeSkillUI.GetRecipeReagentItemLink(recipeId, idx);
      if (reagentLink ~= nil) then
        _, _, reagentId = strfind(reagentLink, "|Hitem:(%d+):");
      else
        reagentId = nil;
      end

      -- Show/Hide the button's texture
      if ((reagentBtns[idx] ~= nil) and
          (reagentBtns[idx]:IsShown()) and
          (textures[idx] ~= nil)) then
        textures[idx]:SetShown(((reagentId ~= nil) and
          (tradeSkillInfo.products[reagentId] ~= nil)));
      end
    end

    -- Add the recipe to the history, if required
    if ((tradeSkillInfo.historyEnabled) and
        (tradeSkillInfo.history ~= nil) and
        (tradeSkillInfo.history:Get() ~= recipeId)) then
      tradeSkillInfo.history:Add(recipeId);
      tradeSkillHistoryBtnEnable();
    end
  else
    -- Hide the textures that indicate a reagent can be created
    for idx = 1, #textures do
      textures[idx]:Hide();
    end
  end

  -- Show/hide the texture that indicates the recipe can be dragged
  tradeSkillInfo.resultTexture:SetShown(
    ((NigglesDBC.settings.ProfGeneralRecipeDrag) and
     (not recipeIsPassive(recipeId))));

  return;
end

--
-- Function to check if a recipe is passive, such as Legion Herbalism and 
-- Mining skills
--
recipeIsPassive = function(recipeId)
  -- Local Variables
  local castTime;

  -- Get the recipe's cast time
  _, _, _, castTime = GetSpellInfo(recipeId or 0);

  return ((castTime or 0) == 0);
end

--
-- Function called when a recipe reagent is clicked. It checks if the reagent
-- can be created by the current trade skill and is known by the current
-- character. If it is, the recipe is selected.
--
local function recipeReagentBtnOnClick(self, mouseButton)
  -- Local Variables
  local reagentId;
  local reagentLink;
  local recipeId = TradeSkillFrame.DetailsFrame.selectedRecipeID;

  -- Check if the button click should be processed
  if ((mouseButton == "LeftButton") and
      (not IsModifierKeyDown()) and
      (recipeId ~= nil)) then
    -- Get a link for the reagent
    reagentLink = C_TradeSkillUI.GetRecipeReagentItemLink(recipeId,
      self.reagentIndex);
    if (reagentLink ~= nil) then
      -- Extract the reagent's ID
      _, _, reagentId = strfind(reagentLink, "|Hitem:(%d+):");

      -- Check if there is a recipe for the reagent
      if (tradeSkillInfo.products[reagentId] ~= nil) then
        -- Show the recipe for the reagent
        tradeSkillRecipeShow(
          bit.band(tradeSkillInfo.products[reagentId],0x00FFFFFF), true);
      end
    end
  end

  return;
end

--
-- Function called when the result icon dragging starts. It picks up the spell
-- for the current recipe, if required
--
local function recipeResultIconOnDragStart(self, mouseButton)
  -- Local Variables
  local recipeId = TradeSkillFrame.DetailsFrame.selectedRecipeID;

  -- Check the feature is enabled and the recipe isn't passive
  if ((NigglesDBC.settings.ProfGeneralRecipeDrag) and
      (not recipeIsPassive(recipeId))) then
    -- Pick up the recipe
    PickupSpell(recipeId);
  end

  return;
end

--
-- Function to check if a trade skill is near or at the skill cap. If the
-- skill cap can be increased, a warning will be issued.
--
local function tradeSkillCheckSkill(skillName, skillRank, skillMaxRank, recipeDifficulty)
  -- Check if the skill is near or at the cap
  if ((NigglesDBC.settings.WarningTradeSkillCap) and
      (recipeDifficulty ~= nil) and
      (recipeDifficulty ~= "") and
      (recipeDifficulty ~= "trivial") and
      (not C_TradeSkillUI.IsNPCCrafting()) and
      (skillMaxRank < PROFESSION_RANKS[#PROFESSION_RANKS][1]) and
      (skillRank >= skillMaxRank-5)) then
    -- Issue a warning
    SELECTED_CHAT_FRAME:AddMessage(format((skillRank >= skillMaxRank and
      L["Trade Skill Capped"] or L["Trade Skill Near Cap"]), skillName))
    PlaySound(SOUNDKIT.RAID_WARNING);
  end

  return;
end

--
-- Function called when the Trade Skill frame is shown. It enables/disables
-- Niggles features, depending on the current settings.
--
tradeSkillFrameOnShow = function(self)
  -- Local Variables
  local button;
  local idx = 1;
  local itemName;
  local reagents = TradeSkillFrame.DetailsFrame.Contents.Reagents
  local texturePath;
  local texture;
  local textures = tradeSkillInfo.textures;

  -- Set up the background texture for the recipe history buttons
  if ((NigglesDBC.settings.ProfGeneralHistory) and
      (tradeSkillInfo.buttonsBg == nil)) then
    texture = TradeSkillFrame.DetailsFrame:CreateTexture(nil, "MEDIUM");
    texture:SetSize(54, 32);
    texture:SetTexture("Interface\\HELPFRAME\\HelpFrameTab-Active");
    texture:SetPoint("BOTTOMLEFT", TradeSkillFrame.DetailsFrame,
      "TOPLEFT", 5, 0);
    tradeSkillInfo.buttonsBg = texture;
  end
  if (tradeSkillInfo.buttonsBg ~= nil) then
    tradeSkillInfo.buttonsBg:SetShown(
      NigglesDBC.settings.ProfGeneralHistory);
  end

  -- Set up the recipe history buttons
  for idx, suffix in ipairs({"PrevPage", "NextPage"}) do
    -- Create the button, if required
    if ((NigglesDBC.settings.ProfGeneralHistory) and
        (tradeSkillInfo.buttons[idx] == nil)) then
      -- Work out the path for the button's textures
      texturePath = "Interface\\Buttons\\UI-SpellbookIcon-"..suffix;

      -- Create the button
      button = CreateFrame("Button", nil, TradeSkillFrame.DetailsFrame);
      button:SetWidth(23);
      button:SetHeight(22);
      button:SetNormalTexture(texturePath.."-Up");
      button:SetPushedTexture(texturePath.."-Down");
      button:SetDisabledTexture(texturePath.."-Disabled");
      button:SetHighlightTexture(
        "Interface\\Buttons\\UI-Common-MouseHilight", "ADD");
      button:SetPoint("BOTTOMLEFT", TradeSkillFrame.DetailsFrame, "TOPLEFT",
        11+((idx-1)*20), 2);
      button:SetID(idx);
      button:SetScript("OnClick", tradeSkillHistoryBtnOnClick);
      button:Disable();
      tradeSkillInfo.buttons[idx] = button;
    else
      button = tradeSkillInfo.buttons[idx];
    end

    -- Show/Hide the button
    if (button ~= nil) then
      button:SetShown(NigglesDBC.settings.ProfGeneralHistory);
    end
  end

  -- Create the textures that indicate creatable recipe reagents, if required
  if (NigglesDBC.settings.ProfGeneralReagents) then
    while (reagents[idx] ~= nil) do
      button = reagents[idx];
      if (textures[idx] == nil) then
        textures[idx] = button:CreateTexture(nil, "BORDER");
        textures[idx]:SetTexture("Interface\\SPELLBOOK\\RotationIconFrame");
        textures[idx]:SetPoint("TOPLEFT", button.Icon, -6, 6);
        textures[idx]:SetPoint("BOTTOMRIGHT", button.Icon, 7, -7);
        textures[idx]:Hide();
      end
      idx = idx+1;
    end
  end

  -- Create the texture that indicates the recipe can be dragged, if required
  if ((NigglesDBC.settings.ProfGeneralReagents) and
      (tradeSkillInfo.resultTexture == nil)) then
    button  = TradeSkillFrame.DetailsFrame.Contents.ResultIcon;
    texture = button:CreateTexture(nil, "OVERLAY");
    texture:SetTexture("Interface\\SPELLBOOK\\RotationIconFrame");
    texture:SetPoint("TOPLEFT", -10, 10);
    texture:SetPoint("BOTTOMRIGHT", 10, -10);
    texture:Show();
    tradeSkillInfo.resultTexture = texture;
  end

  return;
end

--
-- Function to enable/disable the trade skill history buttons
--
tradeSkillHistoryBtnEnable = function()
  -- Enable/disable the history button
  for idx, backward in ipairs({true, false}) do
    if (tradeSkillInfo.buttons[idx] ~= nil) then
      tradeSkillInfo.buttons[idx]:SetEnabled(
        (tradeSkillInfo.history ~= nil) and
        (tradeSkillInfo.history:IsMoveValid(backward)));
    end
  end

  return;
end

--
-- Function called when a Trade skill History button is clicked.
--
tradeSkillHistoryBtnOnClick = function(self, mouseButton)
  -- Local Variables
  local backward = (self:GetID() == 1);

  -- Check the ID of the current trade skill is valid
  if ((tradeSkillInfo.history ~= nil) and
      (tradeSkillInfo.history:IsMoveValid(backward))) then
    tradeSkillRecipeShow(tradeSkillInfo.history:Move(backward), false);
  end

  return;
end

--
-- Function to update the lists of recipes and products of those recipes for
-- the current trade skill
--
tradeSkillInfoUpdate = function()
  -- Local Variables
  local currentInfo = {};
  local currentRank;
  local currentId;
  local numRecipes;
  local productId;
  local productInfo = tradeSkillInfo.products;
  local productLink;
  local strfind = string.find;
  local recipeIds;
  local recipeInfo = tradeSkillInfo.recipes;

  -- Check if the trade skill isn't ready to be interrogated
  if (not C_TradeSkillUI.IsTradeSkillReady()) then
    -- Call the function again in a short while
    C_Timer.After(0.25, tradeSkillInfoUpdate);
    return false;
  end

  -- Get the IDs of all the recipes for the current trade skill
  recipeIds  = C_TradeSkillUI.GetAllRecipeIDs();
  numRecipes = #recipeIds;

  -- Create a list of indices for the trade skill's recipes and their items
  wipe(recipeInfo);
  wipe(productInfo);
  for idx = 1, numRecipes do
    -- Get the item link for the recipe's product
    productLink = C_TradeSkillUI.GetRecipeItemLink(recipeIds[idx]);

    -- Extract the item ID for the product
    _, _, productId = strfind(productLink, "|Hitem:(%d+):");

    -- Save the recipe's product
    recipeInfo[recipeIds[idx]] = (productId or 0);

    -- Save the product's recipe, if the product is an item
    if (productId ~= nil) then
      -- Get info for the recipe
      currentInfo.nextRecipeID     = nil;
      currentInfo.previousRecipeID = nil;
      C_TradeSkillUI.GetRecipeInfo(recipeIds[idx], currentInfo);

      -- Create a recipe ID, adjusted for rank and learned status
      -- NOTE: If there are ever more than 3 ranks of recipe this code will
      --       need to be modified
      currentId = 0x10000000*
        ((currentInfo.previousRecipeID ~= nil and 2 or 0)+
         (((currentInfo.previousRecipeID ~= nil) ~=
          (currentInfo.nextRecipeID ~= nil)) and 1 or 0));
      if (currentInfo.learned) then
        currentId = 0x10000000+currentId;
      else
        currentId = 0x03000000-currentId;
      end
      currentId = currentId+recipeIds[idx];

      -- Save the product's recipe ID, if higher ranked */
      if ((productInfo[productId] or 0) < currentId) then
        productInfo[productId] = currentId;
      end
    end
  end

  return true;
end

--
-- Function to process events
--
local function tradeSkillOnEvent(self, event, ...)
  -- Local Variables
  local _;
  local skillId;

  -- Process the event
  if ((event == "UNIT_SPELLCAST_START") or
      (event == "UNIT_SPELLCAST_CHANNEL_START")) then
    -- Get any gather skill ID for the spell ID
    local _, _, _, _, spellId = ...;
    skillId = gatherSpellSkill[spellId];

    -- Check if a gathering skill is being cast
    if ((skillId ~= nil) and (gatherSkillInfo[skillId] ~= nil)) then
      -- Get the profession's current and max ranks
      local skillName, _, skillRank, skillMaxRank =
        GetProfessionInfo(gatherSkillInfo[skillId].index);

      -- Check the trade skill and issue a warning, if required
      tradeSkillCheckSkill(skillName, skillRank, skillMaxRank,
        tradeSkillTypeFromTooltip(skillId));
    elseif (tradeSkillInfo.recipes[spellId] ~= nil) then
      -- Get info for the recipe
      local recipeInfo = C_TradeSkillUI.GetRecipeInfo(spellId);
      if (recipeInfo ~= nil) then
        -- Get info for the trade skill
        local _, skillName, skillRank, skillMaxRank =
          C_TradeSkillUI.GetTradeSkillLine();

        -- Check the trade skill and issue a warning, if required
        tradeSkillCheckSkill(skillName, skillRank, skillMaxRank,
          recipeInfo.difficulty);
      end
    end
  elseif (event == "SKILL_LINES_CHANGED") then
    -- Initialise some variables
    local skillId;
    local skillRank;
    local professions = {};
    local numProfessions = 0;
    local numMaxed = 0;

    -- Get the character's professions
    professions[1], professions[2], professions[3],
    professions[4], professions[5], professions[6] = GetProfessions();

    -- Get the index of each gather professions the character knows
    for _, professionIdx in pairs(professions) do
      -- Get info for the profession
      _, _, skillRank, _, _, _, skillId = GetProfessionInfo(professionIdx);

      -- Check if the skill is a gather skill
      if (gatherSkillInfo[skillId] ~= nil) then
        -- Save the profession's skill index
        gatherSkillInfo[skillId].index = professionIdx;
      end

      -- Check if the skill is at the maximum rank possible
      numProfessions = numProfessions+1;
      if (skillRank == PROFESSION_RANKS[#PROFESSION_RANKS][1]) then
        numMaxed = numMaxed+1;
      end
    end

    -- Check if there are any professions that aren't maxed
    if (numMaxed < numProfessions) then
      eventFrame:RegisterUnitEvent("UNIT_SPELLCAST_START", "player");
      eventFrame:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_START", "player");
    else
      eventFrame:UnregisterEvent("UNIT_SPELLCAST_START");
      eventFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
    end
  elseif (event == "TRADE_SKILL_DATA_SOURCE_CHANGED") then
    -- Get the ID of the trade skill
    local skillId = C_TradeSkillUI.GetTradeSkillLine();
    if (skillId ~= nil) then
      -- Work out which history list to use
      if ((C_TradeSkillUI.IsTradeSkillGuild()) or
          (C_TradeSkillUI.IsTradeSkillLinked()) or
          (C_TradeSkillUI.IsNPCCrafting())) then
        -- Use a special history list for other characters
        tradeSkillHistory[0]:Clear();
        tradeSkillInfo.history = tradeSkillHistory[0];
      else
        tradeSkillInfo.history = tradeSkillHistory[skillId];
      end

      -- Update the trade skill info
      tradeSkillInfoUpdate();
    else
      tradeSkillInfo.history = nil;
    end

    -- Enable/disable the trade skill history buttons
    tradeSkillHistoryBtnEnable();
  elseif (event == "NEW_RECIPE_LEARNED") then
    tradeSkillInfoUpdate();
  elseif (event == "TRADE_SKILL_HIDE") then
    wipe(tradeSkillInfo.recipes);
    wipe(tradeSkillInfo.products);
  end

  return;
end

--
-- Function to show a specified recipe in the Trade Skill frame. It removes all
-- filtering, un-collapses all categories and makes sure the correct tab is
-- visible. It then selects the recipe and forces it to be visible.
--
tradeSkillRecipeShow = function(recipeId, historyAdd)
  -- Local Variables
  local recipeList = TradeSkillFrame.RecipeList;

  -- Get the ID of the recipe in the appropriate direction
  recipeInfo = C_TradeSkillUI.GetRecipeInfo(recipeId);

  -- Check the recipe's info could be obtained
  if (type(recipeInfo) == "table") then
    -- Close any drop down menus
    CloseDropDownMenus();

    -- Clear all filtering
    C_TradeSkillUI.ClearInventorySlotFilter();
    C_TradeSkillUI.ClearRecipeCategoryFilter();
    C_TradeSkillUI.ClearRecipeSourceTypeFilter();
    C_TradeSkillUI.SetOnlyShowMakeableRecipes(false);
    C_TradeSkillUI.SetOnlyShowSkillUpRecipes(false);
    C_TradeSkillUI.SetRecipeItemNameFilter(nil);
    C_TradeSkillUI.SetRecipeItemLevelFilter(0, 0);
    TradeSkillFrame.SearchBox:SetText("");
    TradeSkillFrame.SearchBox:ClearFocus();

    -- Un-collapse all categories
    recipeList.collapsedCategories = {};

    -- Enable/disable updates to the trade skill history
    tradeSkillInfo.historyEnabled = ((historyAdd == nil) or (historyAdd))

    -- Make sure the correct list is displayed
    if (((recipeInfo.learned) and
         (recipeList.selectedTab ~= LEARNED_TAB)) or
        ((not recipeInfo.learned) and
         (recipeList.selectedTab ~= UNLEARNED_TAB))) then
      -- Change the selected tab
      PanelTemplates_SetTab(recipeList,
        (recipeInfo.learned and LEARNED_TAB or UNLEARNED_TAB));

      -- Update the selected recipes
      C_TradeSkillUI.SetOnlyShowLearnedRecipes(recipeInfo.learned);
      C_TradeSkillUI.SetOnlyShowUnlearnedRecipes(not recipeInfo.learned);
    end

    -- Force a refresh of the list, so the history can be re-enabled
    recipeList:Refresh();
    recipeList:OnUpdate();

    -- Select the correct recipe and make sure it's visible
    recipeList:SelectedAndForceRecipeIDIntoView(recipeInfo.recipeID);

    -- Enable updates to the trade skill history
    tradeSkillInfo.historyEnabled = true;

    -- Update the state of the trade skill history buttons
    tradeSkillHistoryBtnEnable();
  end

  return;
end

--
-- Function hooked to 'SolveArtifact' to capture the spell ID of the artifact
-- being solved.
--
local function tradeSkillSolveArtifact(...)
  -- Local Variables
  local _;
  local skillIdx;

  -- Get the index of the Archaeology trade skill
  _, _, skillIdx = GetProfessions();
  if (skillIdx ~= nil) then
    skillName, _, skillRank, skillMaxRank = GetProfessionInfo(skillIdx);
    tradeSkillCheckSkill(skillName, skillRank, skillMaxRank, "optimal");
  end

  return;
end

--
-- Function to get the skill type from the GameTooltip when a world object is
-- gathered.
--
tradeSkillTypeFromTooltip = function(skillId)
  -- Local Variables
  local colourR;
  local colourG;
  local colourB;
  local skillType = "optimal";

  -- Process each line of the Game Tooltip
  for idx = GameTooltip:NumLines(), 1, -1 do
    -- Check if the tooltip line matches the gather skill's tooltip
    if (_G["GameTooltipTextLeft"..idx]:GetText() ==
         gatherSkillInfo[skillId].tooltip) then
      -- Get the colour of the tooltip line
      colourR, colourG, colourB = _G["GameTooltipTextLeft"..idx]:GetTextColor();

      -- Find the skill type, based on the colour
      for currentType, currentInfo in pairs(tradeSkillTypeInfo) do
        if ((math.abs(colourR-currentInfo.r) < 0.01) and
            (math.abs(colourG-currentInfo.g) < 0.01) and
            (math.abs(colourB-currentInfo.b) < 0.01)) then
          skillType = currentType;
        end
      end
      break;
    end
  end

  return skillType;
end

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to enable/disable the Niggles trade skill features
--
function NigglesTradeSkillEnable()
  -- Local Variables
  local button;
  local idx = 1;

  -- Check if the frame used to monitor events has already be created
  if (eventFrame == nil) then
    -- Create the event frame
    eventFrame = CreateFrame("FRAME", nil);
    eventFrame:SetScript("OnEvent", tradeSkillOnEvent);

    -- Hook the function for Archaeology
    hooksecurefunc("SolveArtifact", tradeSkillSolveArtifact);
  end

  -- (Un)register for events, depending on the current settings
  if (NigglesDBC.settings.ProfGeneralReagents) then
    eventFrame:RegisterEvent("TRADE_SKILL_DATA_SOURCE_CHANGED");
    eventFrame:RegisterEvent("TRADE_SKILL_HIDE");
    eventFrame:RegisterEvent("NEW_RECIPE_LEARNED");
  else
    eventFrame:UnregisterEvent("TRADE_SKILL_DATA_SOURCE_CHANGED");
    eventFrame:UnregisterEvent("TRADE_SKILL_HIDE");
    eventFrame:UnregisterEvent("NEW_RECIPE_LEARNED");
  end
  if (NigglesDBC.settings.WarningTradeSkillCap) then
    eventFrame:RegisterEvent("SKILL_LINES_CHANGED");
    tradeSkillOnEvent(eventFrame, "SKILL_LINES_CHANGED");
  else
    eventFrame:UnregisterEvent("UNIT_SPELLCAST_START");
    eventFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
    eventFrame:UnregisterEvent("SKILL_LINES_CHANGED");
  end

  -- Update the Trade Skill frame, based on current settings
  if ((TradeSkillFrame ~= nil) and
      (TradeSkillFrame:IsShown())) then
    tradeSkillFrameOnShow(TradeSkillFrame);
  end

  -- Update/Clear the recipe info, based on current settings
  if ((NigglesDBC.settings.ProfGeneralReagents) and
      (C_TradeSkillUI.GetTradeSkillLine() ~= nil)) then
    tradeSkillInfoUpdate();
  else
    -- Clear any trade skill recipe info
    wipe(tradeSkillInfo.recipes);
    wipe(tradeSkillInfo.products);
  end

  return;
end

--
-- Function called when Blizzard's Trade Skill addon is loaded
--
function NigglesTradeSkillOnLoad()
  -- Local Variables
  local button;
  local idx = 1;
  local reagentBtns = TradeSkillFrame.DetailsFrame.Contents.Reagents;
  local texture;

  -- Hook the Trade Skill Frame
  TradeSkillFrame:HookScript("OnShow", tradeSkillFrameOnShow);

  -- Hook Blizzard functions
  hooksecurefunc(TradeSkillFrame.DetailsFrame, "RefreshDisplay",
    recipeDetailsOnRefresh);

  -- Hook the 'OnClick' script handler for the reagent buttons
  while (reagentBtns[idx] ~= nil) do
    reagentBtns[idx]:HookScript("OnClick", recipeReagentBtnOnClick);
    idx = idx+1;
  end

  -- Hook the 'OnDragStart' script handler for the result icon
  button = TradeSkillFrame.DetailsFrame.Contents.ResultIcon;
  button:RegisterForDrag("LeftButton");
  button:HookScript("OnDragStart", recipeResultIconOnDragStart);

  return;
end
